//JS linter lines
/*jslint node: true */
'use strict';
/*global Phaser, game, Utils, Globals, SpotObject, MessageHandler*/

/**
 * Main state class
 */

var StateGame1 = function (game) {
  //Configuration data
  this.awake();
  
  //Constant Data
  this.maxCurve = 500;
  this.minCurve = -this.maxCurve;
  this.minX = 0;
  this.constY = game.height * 0.7;
};
StateGame1.prototype = {
  awake : function(){
    this.curve = 300;
    this.step = 0;
    this.stepChange = 10;
    this.minTime = 0;
    this.maxTime = 6;
    this.bulletVelocity = -250;
    this.patternStep = -1;
    this.patternProb = .9;
  },
  create : function (){
    game.stage.backgroundColor = "#88BBFF";
    this.awake();
    game.physics.startSystem(Phaser.Physics.ARCADE);
    
    //left button
    this.btnLeft = game.add.graphics();
    this.btnLeft.beginFill(0xFF0000);
    this.btnLeft.drawRect(0,0,game.width*0.5,game.height);
    this.btnLeft.endFill();
    this.btnLeft.inputEnabled = true;
    this.btnLeft.events.onInputUp.add(this.pressUp, this);
    this.btnLeft.events.onInputDown.add(function(){this.pressDown(-1);}, this);
    this.btnLeft.alpha = 0;
    
    //right button
    this.btnRight = game.add.graphics();
    this.btnRight.beginFill(0x0000FF);
    this.btnRight.drawRect(game.width*0.5,0,game.width*0.5,game.height);
    this.btnRight.endFill();
    this.btnRight.inputEnabled = true;
    this.btnRight.events.onInputUp.add(this.pressUp, this);
    this.btnRight.events.onInputDown.add(function(){this.pressDown(1);}, this);
    this.btnRight.alpha = 0;
    
    //keyboard
    this.cursors = game.input.keyboard.createCursorKeys();
    
    this.sun = game.add.sprite(this.minX,this.constY,'sun');
    game.physics.arcade.enable(this.sun,true);
    this.sun.enableBody = true;
    this.sun.physicsBodyType = Phaser.Physics.ARCADE;
    this.sun.body.velocity.x = 10;
    this.sun.scale.setTo(2);
    
    //clouds initialization
    this.clouds = game.add.group();
    game.physics.arcade.enable(this.clouds,true);
    this.clouds.enableBody = true;
    this.clouds.physicsBodyType = Phaser.Physics.ARCADE;
    this.clouds.createMultiple(10, 'cloud');
	this.clouds.callAll('scale.setTo', 'scale', 2, 2);
    
    //clouds events
    for (var i = 0, len = this.clouds.children.length; i < len; i++) {
      this.clouds.children[i].events.onOutOfBounds.add(this.resetCloud, this);
    }
    
    //Mountain initialization
    this.mountainLeft = game.add.graphics();
    this.mountainRight = game.add.graphics();
    
    //ale initialization
    this.ale = game.add.sprite(game.width*0.25,this.constY - this.curve, 'ale');
    game.physics.arcade.enable(this.ale);
    this.ale.anchor.setTo(0.5,1);
    this.ale.enableBody = true;
    this.ale.physicsBodyType = Phaser.Physics.ARCADE;
    this.ale.scale.setTo(1,1.3);
    
    //steph initialization
    this.steph = game.add.sprite(game.width*0.75,this.constY + this.curve, 'steph');
    game.physics.arcade.enable(this.steph);
    this.steph.anchor.setTo(0.5,1);
    this.steph.enableBody = true;
    this.steph.physicsBodyType = Phaser.Physics.ARCADE;
    this.steph.scale.setTo(1,1.3);
    
    //bullets initialization
    this.bullets = game.add.group();
    game.physics.arcade.enable(this.bullets,true);
    this.bullets.enableBody = true;
    this.bullets.physicsBodyType = Phaser.Physics.ARCADE;
    this.bullets.createMultiple(10, 'bullet');
    
    //score
    var style = {font : 'bold 40px Arial', fill : '#FFFFFF'};
    score = 0;
    this.scoreText = game.add.text(game.width*0.5,50,0,style);
    this.scoreText.anchor.setTo(0.5);
    
    //bullets events
    for (var i = 0, len = this.bullets.children.length; i < len; i++) {
      this.bullets.children[i].events.onOutOfBounds.add(this.resetBullet, this);
    }
    
    //this.bullets.callAll('events.onOutOfBounds.add', 'events.onOutOfBounds', this.resetBullet);
	this.bullets.callAll('anchor.setTo', 'anchor', 0.5, 0.5);
	this.bullets.callAll('scale.setTo', 'scale', 2, 2);
    this.bullets.setAll('checkWorldBounds', true);
    
    game.time.events.add(Phaser.Timer.SECOND * game.rnd.realInRange(this.minTime,this.maxTime), this.fireBullet, this);
    game.time.events.add(Phaser.Timer.SECOND * game.rnd.realInRange(0,10), this.fireClouds, this);
  },
  update : function(){
    if(this.sun.position.x > game.width)
      this.sun.position.x = this.minX;
    this.sun.position.y= this.constY - Math.sin(this.sun.position.x / (game.width*0.5) + 0.5) * game.height * 0.4;
    
    if (game.physics.arcade.overlap (this.ale, this.bullets) || game.physics.arcade.overlap (this.steph, this.bullets))
    {
      if(score > maxScore)
        maxScore = score;
      this.state.start('StateStart');
    }
    
    if(this.cursors.up.isDown)
      this.curve += this.stepChange;
    
    if(this.cursors.down.isDown)
      this.curve -= this.stepChange;
    
    this.scoreText.setText(score);
    
    this.curve += this.step;
    
    if(this.curve > this.maxCurve)
      this.curve = this.maxCurve;
    else if(this.curve < this.minCurve)
      this.curve = this.minCurve;
    
    //Mountain Drawing
    this.mountainLeft.clear();
    this.mountainLeft.beginFill(0x00FF00);
    this.mountainLeft.moveTo(this.minX,this.constY);
    this.mountainLeft.quadraticCurveTo(game.width*0.25,this.constY - this.curve, game.width*0.5, this.constY);
    this.mountainLeft.lineTo(game.width*0.5,game.height);
    this.mountainLeft.lineTo(this.minX,game.height);
    this.mountainLeft.lineTo(this.minX,this.constY);
    this.mountainLeft.endFill();
    
    this.mountainRight.clear();
    this.mountainRight.beginFill(0x00FF00);
    this.mountainRight.moveTo(game.width*0.5,this.constY);
    this.mountainRight.quadraticCurveTo(game.width*0.75,this.constY + this.curve, game.width, this.constY);
    this.mountainRight.lineTo(game.width,game.height);
    this.mountainRight.lineTo(game.width*0.5,game.height);
    this.mountainRight.lineTo(game.width*0.5,this.constY);
    this.mountainRight.endFill();
    
    //ale and steph positions
    var calc = this.mountainLeft.height - this.mountainRight.height;
    this.ale.y = this.constY - calc;
    this.steph.y = this.constY + calc;
  },
  pressUp : function(){
    this.step = 0;
  },
  pressDown : function(dir){
    this.step = dir * this.stepChange;
  },
  resetBullet : function(bullet) {
	bullet.kill();
    score++;
  },
  resetCloud : function(cloud) {
	cloud.kill();
  },
  fireBullet : function(){
    var bullet = this.bullets.getFirstExists(false);
	if (bullet) {
      bullet.reset(game.width, this.constY + Math.sin(game.time.totalElapsedSeconds() * 0.3) * game.height * 0.3 + game.rnd.realInRange(-50,50));
      bullet.body.velocity.x = this.bulletVelocity;
	}
    //Handles the next bullet
    //While not max speed make more faster
    if(this.maxTime > 2){
      this.bulletVelocity *= 1.05;
      this.stepChange *= 1.05;
      this.maxTime *= 0.95;
      this.patternProb *= 0.99;
    }
    var rand = game.rnd.realInRange(this.minTime,this.maxTime);
    if(rand - this.bulletVelocity/screen.height*0.5 < 64)
      rand += 1;
    
    //game.time.events.add(Phaser.Timer.SECOND * rand, this.fireBullet, this);
    
    //Pattern Aditional Handling
    if(game.rnd.realInRange(0,1) < this.patternProb)
      game.time.events.add(Phaser.Timer.SECOND * rand, this.fireBullet, this);
    else
      game.time.events.add(Phaser.Timer.SECOND * rand, this.firePattern, this);
  },
  firePattern : function(){
    if(this.patternStep == -1){
      this.patternStep = 0
      this.pattern = attackPatterns[game.rnd.integerInRange(0,attackPatterns.length-1)]; 
    }
    var bullet = this.bullets.getFirstExists(false);
	if (bullet) {
      bullet.reset(game.width, game.height * this.pattern[this.patternStep].posY);
      bullet.body.velocity.x = this.bulletVelocity;
	}
    this.patternStep++;
    
    var rand = game.rnd.realInRange(this.minTime,this.maxTime);
    if(rand - this.bulletVelocity/screen.height*0.5 < 64)
      rand += 1;
    if(this.patternStep < this.pattern.length)
      game.time.events.add(Phaser.Timer.SECOND * this.pattern[this.patternStep].timer * this.patternProb, this.firePattern, this);
    else{
      this.patternStep = -1;
      game.time.events.add(Phaser.Timer.SECOND * rand, this.fireBullet, this);
    }
  },
  fireClouds : function(){
    var cloud = this.clouds.getFirstExists(false);
	if (cloud) {
      cloud.reset(game.width, this.constY - game.height * 0.5 * game.rnd.realInRange(0,1));
      cloud.body.velocity.x = this.bulletVelocity * 0.5;
	}
    
    var rand = game.rnd.realInRange(0,10);
    //Pattern Aditional Handling
    game.time.events.add(Phaser.Timer.SECOND * rand, this.fireClouds, this);
  }
};