//JS linter lines
/*jslint node: true */
'use strict';
/*global Phaser, game, Utils, Globals, SpotObject, MessageHandler*/

/**
 * Main state class
 */
var score = 0;
var maxScore = 0;

var StateStart = function (game) {
  this.curve = 300;
  this.minX = 0;
  this.constY = game.height * 0.7;
  this.bulletVelocity = -250;
};
StateStart.prototype = {
  create : function (){
    game.stage.backgroundColor = "#88BBFF";
    game.physics.startSystem(Phaser.Physics.ARCADE);
    
    
    this.sun = game.add.sprite(this.minX,this.constY,'sun');
    game.physics.arcade.enable(this.sun,true);
    this.sun.enableBody = true;
    this.sun.physicsBodyType = Phaser.Physics.ARCADE;
    this.sun.body.velocity.x = 20;
    this.sun.scale.setTo(2);
    
    //clouds initialization
    this.clouds = game.add.group();
    game.physics.arcade.enable(this.clouds,true);
    this.clouds.enableBody = true;
    this.clouds.physicsBodyType = Phaser.Physics.ARCADE;
    this.clouds.createMultiple(10, 'cloud');
	this.clouds.callAll('scale.setTo', 'scale', 2, 2);
    
    //clouds events
    for (var i = 0, len = this.clouds.children.length; i < len; i++) {
      this.clouds.children[i].events.onOutOfBounds.add(this.resetCloud, this);
    }
    
    //Mountain initialization
    this.mountainLeft = game.add.graphics();
    this.mountainRight = game.add.graphics();
    
    //ale initialization
    this.ale = game.add.sprite(game.width*0.25,this.constY - this.curve, 'ale');
    game.physics.arcade.enable(this.ale);
    this.ale.anchor.setTo(0.5,1);
    this.ale.enableBody = true;
    this.ale.physicsBodyType = Phaser.Physics.ARCADE;
    this.ale.scale.setTo(1,1.3);
    
    //steph initialization
    this.steph = game.add.sprite(game.width*0.75,this.constY + this.curve, 'steph');
    game.physics.arcade.enable(this.steph);
    this.steph.anchor.setTo(0.5,1);
    this.steph.enableBody = true;
    this.steph.physicsBodyType = Phaser.Physics.ARCADE;
    this.steph.scale.setTo(1,1.3);
    
    var style = {font : 'bold 40px Arial', fill : '#FFFFFF'};
    
    //play button
    this.btnPlay = game.add.text(game.width*0.5,game.height*0.5,"PLAY",style);
    this.btnPlay.anchor.setTo(0.5);
    this.btnPlay.inputEnabled = true;
    this.btnPlay.events.onInputDown.add(function(){this.play();}, this);
    
    //score
    this.scoreText = game.add.text(game.width*0.5,game.height*0.8,"Max Score: " + maxScore,style);
    this.scoreText.anchor.setTo(0.5);
    
    style = {font : 'bold 60px Arial', fill : '#FFFFFF'};
    
    this.title = game.add.text(game.width*0.5,game.height*0.3,"Mountains",style);
    this.title.anchor.setTo(0.5);
    
    style = {font : 'bold 24px Arial', fill : '#FFFFFF'};
    
    this.credits = game.add.text(game.width*0.5,game.height*0.95,"Made by. Alejandro Monascal",style);
    this.credits.anchor.setTo(0.5);
      
    game.time.events.add(Phaser.Timer.SECOND * game.rnd.realInRange(0,10), this.fireClouds, this);
  },
  update : function(){
    if(this.sun.position.x > game.width)
      this.sun.position.x = this.minX;
    this.sun.position.y= this.constY - Math.sin(this.sun.position.x / (game.width*0.5) + 0.5) * game.height * 0.4;
    
    //Mountain Drawing
    this.mountainLeft.clear();
    this.mountainLeft.beginFill(0x00FF00);
    this.mountainLeft.moveTo(this.minX,this.constY);
    this.mountainLeft.quadraticCurveTo(game.width*0.25,this.constY - this.curve, game.width*0.5, this.constY);
    this.mountainLeft.lineTo(game.width*0.5,game.height);
    this.mountainLeft.lineTo(this.minX,game.height);
    this.mountainLeft.lineTo(this.minX,this.constY);
    this.mountainLeft.endFill();
    
    this.mountainRight.clear();
    this.mountainRight.beginFill(0x00FF00);
    this.mountainRight.moveTo(game.width*0.5,this.constY);
    this.mountainRight.quadraticCurveTo(game.width*0.75,this.constY + this.curve, game.width, this.constY);
    this.mountainRight.lineTo(game.width,game.height);
    this.mountainRight.lineTo(game.width*0.5,game.height);
    this.mountainRight.lineTo(game.width*0.5,this.constY);
    this.mountainRight.endFill();
    
    //ale and steph positions
    var calc = this.mountainLeft.height - this.mountainRight.height;
    this.ale.y = this.constY - calc;
    this.steph.y = this.constY + calc;
  },
  play : function(){
    this.state.start('StateGame1');
  },
  resetCloud : function(cloud) {
	cloud.kill();
  },
  fireClouds : function(){
    var cloud = this.clouds.getFirstExists(false);
	if (cloud) {
      cloud.reset(game.width, this.constY - game.height * 0.5 * game.rnd.realInRange(0,1));
      cloud.body.velocity.x = this.bulletVelocity * 0.5;
	}
    
    var rand = game.rnd.realInRange(0,10);
    //Pattern Aditional Handling
    game.time.events.add(Phaser.Timer.SECOND * rand, this.fireClouds, this);
  }
};